/* 
 * Eneso Human Interface Device Library
 * Eneso Tecnología de Adaptación, S.L. 
 */

#include <glib.h>

#ifdef G_OS_UNIX
#ifndef __MACH__

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/inotify.h>
#include <asm/types.h>
#include <linux/hiddev.h>
#include <errno.h>
#include "ehid_linux.h"

static int	fd;
static int	inot_fd;
static int	inot_wd;

static struct hiddev_usage_ref		uref_out;
static struct hiddev_report_info	rinfo_out;


	void 
ehid_linux_init (void)
{
	/* Initialize inotify */
	inot_fd = inotify_init ();

	if (inot_fd < 0)
	{
		perror ("inotify_init");
		return;
	}

	/* Add a watch to the device files folder */
	inot_wd = inotify_add_watch (inot_fd, "/dev/", IN_CREATE);
	
	if (inot_wd < 0)
	{
		perror ("inotify_add_watch");
		return;
	}

	fd = -1;	
}

	void 
ehid_linux_terminate (void)
{
	if (fd >= 0)
		close (fd);

	if (inot_fd >= 0)
	{
		if (inot_wd >= 0)
			inotify_rm_watch (inot_fd, inot_wd);

		close (inot_fd);
	}
}

	void
ehid_linux_receive (void)
{
	struct	hiddev_usage_ref ur [32];
	struct	timeval tv;
	fd_set	rdfs;
	int		n_bytes;
	int 	n_reports;
	int 	i;

	if (fd >= 0)
	{
		/* Check whether read() will block */
		FD_ZERO (&rdfs);
		FD_SET (fd, &rdfs);

		tv.tv_sec = 0;
		tv.tv_usec = 100e3;

		if (select (fd + 1, &rdfs, NULL, NULL, &tv) == 0)
			return;

		/* Read the file descriptor */
		n_bytes = read (fd, &ur, sizeof (struct hiddev_usage_ref) * 32);

		if (n_bytes < 0)
		{
			/* Something failed. Probably the device was disconnected */
			close (fd);
			fd = -1;
			check_connected_and_notify ();
			return;
		}

		/* Read the reports */
		n_reports = n_bytes / sizeof (struct hiddev_usage_ref);

		for (i = 0; i < n_reports; i++)
		{
			read_buffer [ur [i].usage_index] = ur [i].value;

			if (ur [i].usage_index == user_input_length - 2)
			{
				if (user_message_cb) user_message_cb (read_buffer, user_input_length - 1);
				break;
			}
		}
	}
}

	void 
ehid_linux_send (const gchar* message)
{
	int i;

	for (i = 0; i < user_output_length - 1; i++)
	{
		uref_out.usage_index = i;
		uref_out.value = message [i];

		ioctl (fd, HIDIOCSUSAGE, &uref_out);
	}

	ioctl (fd, HIDIOCSREPORT, &rinfo_out);
}

	void
ehid_linux_search (void)
{
	struct inotify_event*	event;
	struct timeval			tv;
	char	buffer	[sizeof (struct inotify_event) * 10];
	fd_set	rdfs;

	if (status == CONNECTED)
		return;

	FD_ZERO (&rdfs);
	FD_SET (inot_fd, &rdfs);

	tv.tv_sec	= 0;
	tv.tv_usec	= 100e3;

	if (select (inot_fd + 1, &rdfs, NULL, NULL, &tv) > 0)
	{
		if (read (inot_fd, buffer, sizeof (buffer)) > 0)
			check_connected_and_notify ();
	}
}

	ConnectionStatus
ehid_linux_check (void)
{
	struct hiddev_devinfo		devinfo;
	struct hiddev_field_info	finfo;
	struct hiddev_usage_ref		uref;
	gchar		path [50] = "/dev/";
	guint		dev, i;
	gboolean	connected = FALSE;

	strcat (path, user_name);

	/* Try to open the hiddev file */
	if ((fd = open (path, O_RDONLY)) < 0)
		return DISCONNECTED;

	/* Check the vendor and product ids */
	ioctl (fd, HIDIOCGDEVINFO, &devinfo);

	if ((user_vid == 0 || user_vid == (devinfo.vendor & 0xFFFF)) &&
			(user_pid == 0 || user_pid == (devinfo.product & 0xFFFF)))
	{
		/* Report */
		rinfo_out.report_type = HID_REPORT_TYPE_OUTPUT;
		rinfo_out.report_id = HID_REPORT_ID_FIRST;

		if (ioctl (fd, HIDIOCGREPORTINFO, &rinfo_out) < 0)
			return DISCONNECTED;

		/* Field */
		finfo.report_type = rinfo_out.report_type;
		finfo.report_id = rinfo_out.report_id;
		finfo.field_index = 0;

		if (ioctl (fd, HIDIOCGFIELDINFO, &finfo) < 0)
			return DISCONNECTED;

		/* Usages */
		for (i = 0; i < finfo.maxusage; i++)
		{
			uref.report_type = finfo.report_type;
			uref.report_id = finfo.report_id;
			uref.field_index = 0;
			uref.usage_index = i;

			if (ioctl (fd, HIDIOCGUCODE, &uref) < 0)
				return DISCONNECTED;

			if (ioctl (fd, HIDIOCGUSAGE, &uref) < 0)
				return DISCONNECTED;

			if (i == 0)
				memcpy (&uref_out, &uref, sizeof (uref));
		}

		/* Make read() return hiddev_usage_ref type structures */
		i = HIDDEV_FLAG_UREF;
		if (ioctl (fd, HIDIOCSFLAG, &i) < 0)
			return DISCONNECTED;

		connected = TRUE;
	}
	else
	{
		close (fd);
		fd = -1;
		connected = FALSE;
	}

	return (connected ? CONNECTED : DISCONNECTED);
}

#endif /* __MACH__ */
#endif /* G_OS_UNIX */
