/* 
 * Eneso Human Interface Device Library
 * Eneso Tecnología de Adaptación, S.L. 
 */

#include <glib.h>

#ifdef G_OS_WIN32

#define _WIN32_WINDOWS 	0x0501
#define WINVER 			0x0501

#include <windows.h>
#include <setupapi.h>
#include <dbt.h>
#include <usb.h>
#include <hidsdi.h>
#include "ehid_windows.h"

#define DBT_DEVICEARRIVAL 			0x8000
#define DBT_DEVICEREMOVECOMPLETE	0x8004

HIDAPI BOOLEAN NTAPI HidD_GetPreparsedData (HANDLE, PHIDP_PREPARSED_DATA  *);
HIDAPI BOOLEAN NTAPI HidD_FreePreparsedData (PHIDP_PREPARSED_DATA);

static HDEVNOTIFY 	dev_notify;
static OVERLAPPED 	overlapped;
static HANDLE 		event;

static HANDLE 	handle_read		= NULL;
static HANDLE 	handle_write	= NULL;

static LRESULT CALLBACK wndproc (HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam);


	void 
ehid_windows_init (void)
{
	WNDCLASS	class = {0};
	HWND		window;
	GUID		guid;
	DEV_BROADCAST_DEVICEINTERFACE filter;

	/* Create a window */
	class.lpszClassName		= TEXT ("ehid");
	class.hInstance 		= NULL;
	class.lpfnWndProc		= wndproc;

	RegisterClass (&class);

	window = CreateWindowEx (0, class.lpszClassName, TEXT ("ehid"),
			WS_OVERLAPPEDWINDOW, 0, 0, 100, 100, NULL, NULL, NULL, NULL);

	/* Register to device connection/disconnection notifications */
	HidD_GetHidGuid (&guid);

	ZeroMemory (&filter, sizeof (filter));

	filter.dbcc_size = sizeof (DEV_BROADCAST_DEVICEINTERFACE);
	filter.dbcc_devicetype = DBT_DEVTYP_DEVICEINTERFACE;
	filter.dbcc_classguid = guid;

	dev_notify = RegisterDeviceNotification (window, &filter, 0);
}

	void 
ehid_windows_terminate (void)
{
	handle_write = NULL;
	handle_read = NULL;

	UnregisterDeviceNotification (dev_notify);
}

	void
ehid_windows_receive (void)
{
	if (handle_read)
	{
		DWORD n_bytes;

		ReadFile (handle_read, read_buffer, user_input_length, &n_bytes, &overlapped);

		switch (WaitForSingleObject (event, 100))
		{
			case WAIT_OBJECT_0:
				GetOverlappedResult (handle_read, &overlapped, &n_bytes, 0);
				break;

			case WAIT_TIMEOUT: 
				CancelIo (handle_read);
				break;

			default: break;
		}

		if (n_bytes > 0)
		{
			if (user_message_cb)
				user_message_cb (read_buffer + 1, n_bytes - 1);
		}
	}
}

	void 
ehid_windows_send (const gchar* message)
{
	DWORD	n_bytes;
	gchar*	report;
	int		i;

	if (!handle_write)
		return;

	report = g_malloc (user_output_length);

	report [0] = 0;
	for (i = 1; i < user_output_length; i++) report [i] = message [i - 1];

	n_bytes = 0;

	WriteFile (handle_write, report, user_output_length, &n_bytes, NULL);

	g_free (report);
}

	ConnectionStatus
ehid_windows_check (guint* vid_found, guint* pid_found)
{
	SP_DEVICE_INTERFACE_DATA			interface_data;
	PSP_DEVICE_INTERFACE_DETAIL_DATA	interface_details;
	HIDD_ATTRIBUTES						attributes;
	GUID		guid;
	HDEVINFO	devices;
	HANDLE		handle;
	int			index;
	LONG		size;
	LONG		request;
	BOOL		result;
	gboolean	connected = FALSE;

	/* Get the interface id for HID devices */
	HidD_GetHidGuid (&guid);

	/* Retrieve a list with all the connected HID devices */
	devices = SetupDiGetClassDevs (&guid, NULL, NULL, DIGCF_PRESENT | DIGCF_INTERFACEDEVICE);
	if (devices == INVALID_HANDLE_VALUE)
		return FALSE;

	interface_data.cbSize = sizeof (interface_data);

	/* Traverse the list, looking for the requested devices */
	handle_read = NULL;
	handle_write = NULL;

	index = 0;

	while (1)
	{
		/* Extract the information about a device in the list */
		result = SetupDiEnumDeviceInterfaces (devices, NULL, &guid, index, 
				&interface_data);
		if (!result)
			break;

		/* Get the path to the device. The first call is to get the size of the
		 * returned structure */
		SetupDiGetDeviceInterfaceDetail (devices, &interface_data, NULL,
				0, &size, NULL);

		interface_details = (PSP_DEVICE_INTERFACE_DETAIL_DATA) g_malloc (size);
		interface_details->cbSize = sizeof (SP_DEVICE_INTERFACE_DETAIL_DATA);

		SetupDiGetDeviceInterfaceDetail (devices, &interface_data, interface_details,
				size, &request, NULL);

		event = CreateEvent (NULL, 0, 0, "");

		overlapped.hEvent = event;
		overlapped.Offset = 0;
		overlapped.OffsetHigh = 0;

		/* Create a handle for the device */
		handle = CreateFile (interface_details->DevicePath, 
				GENERIC_READ | GENERIC_WRITE, 
				FILE_SHARE_READ | FILE_SHARE_WRITE,
				(LPSECURITY_ATTRIBUTES) NULL,
				OPEN_EXISTING, 0, NULL);

		/* If we cannot open the file with read and write permissions it probably is
		 * a mouse or a keyboard, we are not interested. */
		if (handle == INVALID_HANDLE_VALUE)
		{
			free (interface_details);
			index++;
			continue;
		}

		/* Read its attributes */
		attributes.Size = sizeof (attributes);
		HidD_GetAttributes (handle, &attributes);

		/* If the vendor and products ids match keep looking */
		if ((user_vid == 0 || attributes.VendorID == user_vid) &&
				(user_pid == 0 || attributes.ProductID == user_pid))
		{
			PHIDP_PREPARSED_DATA	data;
			HIDP_CAPS				caps;

			HidD_GetPreparsedData (handle, &data);
			HidP_GetCaps (data, &caps);

			/* If input and output report sizes match, it is the device we are 
			 * looking for */
			if (caps.InputReportByteLength == user_input_length &&
					caps.OutputReportByteLength == user_output_length) 
			{
				handle_write = handle;
				handle_read = CreateFile (interface_details->DevicePath,
						GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE, 
						(LPSECURITY_ATTRIBUTES) NULL,
						OPEN_EXISTING, FILE_FLAG_OVERLAPPED, NULL);

				if (handle_read == INVALID_HANDLE_VALUE)
					return FALSE;

				*vid_found	= attributes.VendorID;
				*pid_found	= attributes.ProductID;

				connected = TRUE;
			}

			HidD_FreePreparsedData (data);
			free (interface_details);

			if (handle_read)
				break;
		}

		CloseHandle (handle);

		index++;
	}

	SetupDiDestroyDeviceInfoList (devices);

	return (connected ? CONNECTED : DISCONNECTED);
}

	static LRESULT CALLBACK 
wndproc (HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	if (uMsg == WM_DEVICECHANGE)
	{
		if (wParam == DBT_DEVICEARRIVAL ||	wParam == DBT_DEVICEREMOVECOMPLETE)
		{
			/* Windows is notifying a device connection or disconnection, check whether
			 * it is ours */
			check_connected_and_notify ();
		}

		return 0;
	}

	return DefWindowProc (hwnd, uMsg, wParam, lParam);
} 

#endif /* G_OS_WIN32 */
