/* 
 * enPathia
 * Eneso Tecnología de Adaptación, S.L. 
 */

#include <glib.h>
#include <glib/gi18n.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "ehid.h"
#include "enpathia.h"
#include "keyboard.h"
#include "menu.h"
#include "misc.h"
#include "profile.h"
#include "protocol.h"
#include "action.h"

gchar* 				action_text [N_ACTIONS];
cairo_surface_t*	action_image [N_ACTIONS][3];
cairo_surface_t*	action_more_image [3];

gboolean dragging = FALSE;


	void
init_actions (void)
{
	gchar*	image_file;

	action_text	[0] = g_strdup (_("Cancel"));
	action_text [1] = g_strdup (_("Keyboard"));
	action_text [2] = g_strdup (_("Click"));
	action_text [3] = g_strdup (_("Launch"));
	action_text [4] = g_strdup (_("Options"));
	action_text [5] = g_strdup (_("Drag"));
	action_text [6] = g_strdup (_("Drop"));

#define LOAD_IMAGE(action,file)	\
	image_file = get_data_file ("/images/menu/" file "_0.png"); \
	action_image [action][0] = cairo_image_surface_create_from_png (image_file); \
	g_free (image_file);	\
	image_file = get_data_file ("/images/menu/" file "_1.png"); \
	action_image [action][1] = cairo_image_surface_create_from_png (image_file);	\
	g_free (image_file);	\
	image_file = get_data_file ("/images/menu/" file "_2.png"); \
	action_image [action][2] = cairo_image_surface_create_from_png (image_file); \
	g_free (image_file);

	LOAD_IMAGE (0, "cancel");
	LOAD_IMAGE (1, "keyboard");
	LOAD_IMAGE (2, "click");
	LOAD_IMAGE (3, "execute");
	LOAD_IMAGE (4, "options");
	LOAD_IMAGE (5, "drag");
	LOAD_IMAGE (6, "drop");

#undef LOAD_IMAGE
	
	image_file = get_data_file ("/images/menu/more_0.png");
	action_more_image [0] = cairo_image_surface_create_from_png (image_file); 
	g_free (image_file);

	image_file = get_data_file ("/images/menu/more_1.png");
	action_more_image [1] = cairo_image_surface_create_from_png (image_file); 
	g_free (image_file);

	image_file = get_data_file ("/images/menu/more_2.png");
	action_more_image [2] = cairo_image_surface_create_from_png (image_file); 
	g_free (image_file);

}

	void
destroy_actions (void)
{
	gint	i, j;

	for (i = 0; i < N_ACTIONS; i++)
	{
		g_free (action_text [i]);

		for (j = 0; j < 3; j++)
			cairo_surface_destroy (action_image [i][j]);
	}

	for (j = 0; j < 3; j++)
		cairo_surface_destroy (action_more_image [j]);
}

	void
execute_action (Action action)
{
	switch (action)
	{
		case CANCEL:
			/* Cancel *
			 * Keep moving, with the proper menu */
			if (dragging)
				change_menu (&menu_dragging);
			break;

		case KEYBOARD:
			/* Keyboard *
			 * Open the keyboard */
			show_keyboard (current_profile.keyboard.x, current_profile.keyboard.y);
			break;

		case CLICK:
			/* Click *
			 * Left click */
			send_command (COMMAND_CLICK, CLICK_LEFT, 0);
			break;

		case EXECUTE:
			/* Execute *
			 * Double click */
			send_command (COMMAND_CLICK, CLICK_DOUBLE, 0);
			break;

		case OPTIONS:
			/* Option *
			 * Right click */
			send_command (COMMAND_CLICK, CLICK_RIGHT, 0);
			break;

		case DRAG:
			/* Drag *
			 * Press, change menu and start moving */
			send_command (COMMAND_CLICK, CLICK_PRESS, 0);
			dragging = TRUE;
			change_menu (&menu_dragging);
			break;

		case DROP:
			/* Drop *
			 * Release the mouse button */
			send_command (COMMAND_CLICK, CLICK_RELEASE, 0);
			dragging = FALSE;
			change_menu (&menu_moving);
			break;

		default: break;
	}

	change_mode (MODE_MOVEMENT);
}
